@@ -9,13 +9,15 @@ from pyjon.utils.main import create_function
 
 from pyf.componentized.components import BaseAdapter
 from pyf.componentized.configuration.keys import RepeatedKey, CompoundKey,\
-    SimpleKey
+    SimpleKey, BooleanKey
 from pyf.componentized.configuration.fields import InputField, SingleSelectField
+from pyf.transport.packets import Packet
 
 class ComputeAttributes(BaseAdapter):
     name = "compute_attributes"
     
-    configuration = [RepeatedKey('attributes', 'attribute',
+    configuration = [BooleanKey('cache_values', label="Cache values", default=False),
+                     RepeatedKey('attributes', 'attribute',
                                  content=CompoundKey('attribute',
                                                      text_value='getter',
                                                      attributes={'name': 'name',
@@ -59,7 +61,7 @@ class ComputeAttributes(BaseAdapter):
                                               % (name,
                                                  self.name,
                                                  str(e)), message_type="error")
-                        raise e
+                        raise
                 
                 self.getters[getter_name] = create_function(getter,
                                                             args=[getter_code,
@@ -86,7 +88,7 @@ class ComputeAttributes(BaseAdapter):
                         self.message_callback("Error on attribute %s of %s: %s"\
                                               % (name,
                                                  self.name,
-                                                 str(e)))
+                                                 str(e)), message_type="error")
                         raise e
                 
                 self.getters[getter_name] = create_function(getter,
@@ -98,8 +100,9 @@ class ComputeAttributes(BaseAdapter):
     
     @component('IN', 'OUT')
     def launch(self, values, out):
+        cache_values = self.get_config_key('cache_values', False)
         for value in values:
-            value = DynamicObjectAdapter(value)
+            value = DynamicObjectAdapter(value, cache_getters=cache_values)
             
             for key, getter in self.getters.iteritems():
                 value.add_attribute_getter(key, getter)
@@ -200,12 +203,14 @@ def scrub_sa_object(item, **kwargs):
     
             
 class DynamicObjectAdapter(object):
-    def __init__(self, former_object):
+    def __init__(self, former_object, cache_getters=False):
         self.former_object = former_object
         self.scrubbed = False
         self.__attributes = dict()
         self.__attribute_getters = dict()
         
+        self.__cache_getters = cache_getters
+        
         self.__initialized = True
         
     def add_attribute_getter(self, key, getter):
@@ -221,7 +226,11 @@ class DynamicObjectAdapter(object):
         self.former_object = new_item
         
     def compute_getter(self, key):
-        return self.__attribute_getters[key](self, self.former_object)
+        value = self.__attribute_getters[key](self, self.former_object)
+        if self.__cache_getters:
+            self.add_attribute_value(key, value)
+        
+        return value
     
     def __hasattr__(self, key):
         return (key in self.__attributes
@@ -243,3 +252,19 @@ class DynamicObjectAdapter(object):
             object.__setattr__(self, attribute, value)
         else:
             self.add_attribute_value(attribute, value)
+    
+    def serialize(self):
+        obj_attrs = dict()
+        if hasattr(self.former_object, 'keys') and hasattr(self.former_object, 'items'):
+            obj_attrs = dict(self.former_object.values())
+        else:
+            obj_attrs = dict(self.former_object.__dict__.items())
+        
+        obj_attrs.update(self.__attributes)
+        obj_attrs.update([(key, self.compute_getter(key)) for key, value in self.__attribute_getters])
+        
+        return Packet(obj_attrs).serialized
+    
+    @classmethod
+    def deserialize(cls, value_dict):
+        return Packet.deserialize(value_dict)
\ No newline at end of file
